;**************************************
; DIPOLE (2 VOICE) IRQ PLAYER FOR COCO 
; (C) SIMON JONASSEN 2017
;**************************************
		opt	6809
		opt	cd
		org	$5800
start		orcc	#$50

;**************************************
; double note values to save shifts
; on the sequencer
;**************************************

		ldx	#zix
convert		ldd	,x
		asla
		aslb	
		std	,x++
		cmpx	#endzix
		bne	convert
;**************************************
; SOUND ON DAC
;**************************************
		lda	$ff01			; Select DAC as sound source
		anda	#$f7
		sta	$ff01
		lda	$ff03
		anda	#$f7
		sta	$ff03

		lda	$ff23
		ora	#8
		sta	$ff23
;**************************************
; DP for note routine speed
;**************************************
		lda	#note/256
		tfr	a,dp
;**************************************
; setup vectored IRQ using HSYNC
;**************************************
		lda	#$7e
		sta	$10c
		ldx	#note
		stx	$10d
;**************************************
; NUKE VSYNC IRQ
;**************************************
		lda	$ff03
		anda	#$fe
		sta	$ff03
;**************************************
; ENABLE HSYNC IRQ
;**************************************
		lda	$ff01
		ora	#$3
		sta	$ff01
		lda	$ff00	
;**************************************
; CLEAR IRQ MASK ON THE CPU
;**************************************
		andcc	#$ef
;**************************************
; HANG ABOUT AND DO NUTHIN
; YOUR CODE GOES HERE SPUDZ !!
;**************************************
		RTS


;**************************************
; 2 VOICE NOTE ROUTINE
;**************************************
		opt	cc
		opt	ct
note
		
sum		ldd 	#$0000		;CH1 OSCILLATOR
freq		addd 	#$0000 
		std	<sum+1
sum2		ldd 	#$0000		;CH2 OSCILLATOR
freq2		addd 	#$0000 
		std 	<sum2+1		;we have c2 in A - so add chan1
bb		adda	<sum+1		;carry just incase
		rora			;ch1+ch2/2
		sta	$ff20		;OUT ON DAC

		dec	<syncount	
		bne	out
;**************************************
; SEQUENCER
;**************************************
		opt	cc
		opt	ct
play2

oldx		ldx	#try		;save pattern position
curnote		ldd	,x++		;load 2 notes from pattern
		cmpa	#$ff
		bne	plnote		;pattern ends with $ff 
		ldx	#zix		;we looped
		ldd	,x++
plnote		stx	<oldx+1		;restore pattern position to start
		sta	<frq1+2
		stb	<frq2+2
frq1		ldx	#freqtab	;get the right freq
		ldx	,x
		stx	<freq+1		;store
frq2		ldx	#freqtab
		ldx	,x
		stx	<freq2+1
out		lda	$ff00		;ack irq
		rti


;**************************************
; IRQ COUNTER
;**************************************
syncount	fcb	0	
;**************************************
;equal tempered 12 note per 
;octave frequency table (7.875 Khz)
;**************************************
		align	$100

freqtab	fdb	0,282,298,316,335,355,376,398,422,447,474,502
	fdb	532,563,597,632,670,710,752,796,844,894,947,1003	
	fdb	1063,1126,1193,1264,1339,1419,1503,1593,1688,1788,1894,2007
	fdb	2126,2253,2387,2529,2679,2838,3007,3186,3375,3576,3789,4014
	fdb	4252,4505,4773,5057,5358,5676,6014,6371,6750,7152,7577,8028
	fdb	8505,9011,9546,10114,10716,11353,12028,12743,13501,14303,15154,16055
	fdb	17010,18021,19093,20228,21431,22705,24056,25486,27001,28607,30308,32110
	fdb	17010*2,18021*2,19093*2,20228*2,21431*2,22705*2,24056*2,25486*2,27001*2,28607*2,30308*2,32110*2

;**************************************
; DA ZIX (ONE LONG PATTERN) 
; END WITH $80
;**************************************
zix
	include	"music.asm"
endzix	fcb	$ff
		
		end		start